package bikerent.model;

import java.time.Duration;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.time.format.FormatStyle;
import java.util.Locale;

public class MyRent implements Rent {
	private LocalDateTime inizio, fine;
	private Rate rate;
	
	public MyRent(LocalDateTime inizio, LocalDateTime fine, Rate rate) {
		this.inizio = inizio;
		this.fine = fine;
		this.rate = rate;
	}
	
	public boolean isRegular() {
		if (rate.getDurataMax().isPresent()) {
			Duration durata = Duration.between(inizio, fine);
			Duration durataMax = rate.getDurataMax().get();
			return durata.toMinutes()<=durataMax.toMinutes();
		}
		else
		if (rate.getOrarioMax().isPresent() ) {
			LocalDateTime omax = LocalDateTime.of(inizio.toLocalDate(), rate.getOrarioMax().get());
			return fine.isBefore(omax) || fine.equals(omax);
		}
		else throw new IllegalArgumentException("Invalid Rate");
	}
	
	/*
	 public boolean isRegular2() {
	 	Duration durata = Duration.between(inizio, fine);
		Optional<LocalDateTime> termine = rate.getOrarioMax().isPresent() ? Optional.of(LocalDateTime.of(inizio.toLocalDate(),rate.getOrarioMax().get())) : Optional.empty();
		Optional<Boolean> ok1 = rate.getDurataMax().map(dmax -> durata.toMinutes()<=dmax.toMinutes());
		Optional<Boolean> ok2 = termine.map(omax -> (fine.isBefore(omax) || fine.equals(omax)));
		return ok1.orElse(ok2.orElse(null));
	}
	*/

	public LocalDateTime getStart() {
		return inizio;
	}
	
	public LocalDateTime getEnd() {
		return fine;
	}
	
	@Override
	public String toString() {
		return "Noleggio \ndalle " + 
				inizio.toLocalTime().format(DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY)) +
				" di " + 
				inizio.toLocalDate().format(DateTimeFormatter.ofLocalizedDate(FormatStyle.FULL).withLocale(Locale.ITALY)) +
				"\n" +
				" alle " +
				fine.toLocalTime().format(DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY)) +
				" di " + 
				fine.toLocalDate().format(DateTimeFormatter.ofLocalizedDate(FormatStyle.FULL).withLocale(Locale.ITALY)) +
				"\n" +
				"Durata totale: " + toStringDuration(Duration.between(inizio, fine));
		}
	
	private String toStringDuration(Duration duration) {
		int minuti = duration.toMinutesPart();
		String sMinuti = (minuti<10 ? "0" : "") + minuti;
		return duration.toHours() + ":" + sMinuti;
	}
	
}
