package bikerent.persistence;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.time.Duration;
import java.time.LocalTime;
import java.util.HashMap;
import java.util.Map;
import java.util.Optional;

import bikerent.model.Periodo;
import bikerent.model.Rate;

public class MyRateReader implements RateReader
{
	@Override
	public Map<String,Rate> readRates(Reader reader) throws IOException, BadFileFormatException {
		Map<String,Rate> mappaTariffe = new HashMap<>();
		Rate rate;
		BufferedReader fin = new BufferedReader(reader);
		while ((rate = readOneRate(fin)) != null) {
			// System.out.println("Letta tariffa " + rate.getCitta());
			mappaTariffe.put(rate.getCitta(), rate);
		}	
		return mappaTariffe;
	}
	
	private Rate readOneRate(BufferedReader reader) throws IOException, BadFileFormatException
	{
		// CITTA, NN cent per NN minuti, poi NN cent per NN minuti, max NN ore, sanzione X.YY euro
		// CITTA, NN cent per NN minuti, poi NN cent per NN minuti, max entro HH:MM, sanzione X.YY euro
		String line;
		try {			
			while ((line = reader.readLine())!=null) {
				String[] items = line.split(",");
				// items[0] = "CITTA"
				// items[1] = "NN cent per NN minuti"
				// items[2] = "poi NN cent per NN minuti"
				// items[3] = "max NN ore" OPPURE "max entro HH:MM"
				// items[4] = "sanzione X.YY euro"
				//
				// for (String ss : items) System.out.println(ss);
				//
				String citta = items[0].trim();
				Periodo primo = estraiPrimoPeriodo(items[1].trim());
				Periodo successivi = estraiSuccessiviPeriodi(items[2].trim());
				Optional<Duration> durataMax = estraiDurataMax(items[3].trim());
				Optional<LocalTime> orarioMax = estraiOrarioMax(items[3].trim());
				double sanzione = estraiSanzione(items[4].trim());
				return new Rate(citta, primo, successivi, durataMax, orarioMax, sanzione);
			}
		}
		catch (IOException | NumberFormatException e)
		{
			throw new BadFileFormatException(e);
		}
		return null; // will never get here
	}

	
	// "sanzione X.YY euro"
	private double estraiSanzione(String s) throws BadFileFormatException {
		String[] elements = s.split("\\s+");
		if (elements.length!=3) throw new BadFileFormatException("formato errato, sono richiesti tre campi");
		if (!elements[0].equalsIgnoreCase("sanzione")) throw new BadFileFormatException("manca la keyword 'sanzione'");
		if (!elements[2].equalsIgnoreCase("euro")) throw new BadFileFormatException("manca la keyword 'euro'");
		return Double.parseDouble(elements[1]);
	}

	//  "max entro HH:MM"
	private Optional<LocalTime> estraiOrarioMax(String s) throws BadFileFormatException {
		String[] elements = s.split("\\s+");
		if (elements.length!=3) throw new BadFileFormatException("formato errato, sono richiesti tre campi");
		if (!elements[0].equalsIgnoreCase("max")) throw new BadFileFormatException("manca la keyword 'max'");
		if (!elements[1].equalsIgnoreCase("entro")) return Optional.empty();
		return Optional.of(LocalTime.parse(elements[2]));
	}

	// "max NN ore"
	private Optional<Duration> estraiDurataMax(String s) throws BadFileFormatException {
		String[] elements = s.split("\\s+");
		if (elements.length!=3) throw new BadFileFormatException("formato errato, sono richiesti tre campi");
		if (!elements[0].equalsIgnoreCase("max")) throw new BadFileFormatException("manca la keyword 'max'");
		if (!elements[2].equalsIgnoreCase("ore")) return Optional.empty();
		return Optional.of(Duration.ofHours(Integer.parseInt(elements[1])));
	}

	
	// items[1] = "NN cent per NN minuti"
	private Periodo estraiPrimoPeriodo(String s) throws BadFileFormatException {
		String[] elements = s.split("\\s+");
		// for (int i=0; i<elements.length; i++) System.out.println(i + " " + elements[i]);
		return estraiPeriodo(elements, 0);
	}

	// items[2] = "poi NN cent per NN minuti"
	private Periodo estraiSuccessiviPeriodi(String s) throws BadFileFormatException {
		String[] elements = s.split("\\s+");
		// for (int i=0; i<elements.length; i++) System.out.println(i + " " + elements[i]);
		if (elements[0].equalsIgnoreCase("poi")) return estraiPeriodo(elements, 1);
		else throw new BadFileFormatException("manca la keyword 'max'");
	}

	
	// items[1] = "NN cent per NN minuti"
	private Periodo estraiPeriodo(String[] elements, int index) throws BadFileFormatException {
		if (elements.length-index != 5) throw new BadFileFormatException("sono richiesti cinque campi, non " + (elements.length-index));
		if (!elements[index+1].equalsIgnoreCase("cent")) throw new BadFileFormatException("manca la keyword 'cent'");
		if (!elements[index+2].equalsIgnoreCase("per")) throw new BadFileFormatException("manca la keyword 'per'");
		if (!elements[index+4].equalsIgnoreCase("minuti")) throw new BadFileFormatException("manca la keyword 'minuti'");
		return new Periodo(Integer.parseInt(elements[index+0]), Duration.ofMinutes(Integer.parseInt(elements[index+3])));
	}	

}
