package bikerent.ui.javafx;

import java.text.NumberFormat;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeParseException;
import java.time.format.FormatStyle;
import java.util.Locale;

import bikerent.ui.controller.Controller;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.DatePicker;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.control.TextField;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.scene.text.Font;
import javafx.scene.text.FontWeight;

public class RentABikePane extends BorderPane {

	private TextArea outputArea;
	private Controller controller;
	private Button bottoneTicket;
	private VBox leftPane, centerPane;
	private ComboBox<String> combo; 
	private DatePicker pickerInizio, pickerFine;
	private TextField fieldInizio, fieldFine;
	private DateTimeFormatter formatter;
	
	public RentABikePane(Controller controller) {
		this.controller = controller;
		formatter = DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY);
		//
		HBox topPane = new HBox();
		ObservableList<String> nomiTariffe = this.controller.getCityNames();
		combo = new ComboBox<>(nomiTariffe);
		combo.setValue(nomiTariffe.get(0));
		//
		bottoneTicket = new Button("Calcola costo");
		bottoneTicket.setStyle("-fx-background-color: red;");
		bottoneTicket.setOnAction(this::calcolaCostoNoleggio);
		//
		topPane.getChildren().addAll(new Label("Città:  "), combo, new Label("        "), bottoneTicket);
		this.setTop(topPane);
		//
		leftPane = new VBox();
		leftPane.setPrefWidth(150);
		leftPane.getChildren().add(new Label(" Inizio noleggio: "));
		pickerInizio = new DatePicker(LocalDate.now()); 
		fieldInizio = new TextField();
		fieldInizio.setText(LocalTime.now().format(formatter));
		fieldInizio.setPromptText("Inserire ora iniziale");
		leftPane.getChildren().addAll(pickerInizio,fieldInizio);
		this.setLeft(leftPane);
		//
		centerPane = new VBox();
		centerPane.setPrefWidth(150);
		centerPane.getChildren().add(new Label(" Fine noleggio: "));
		pickerFine = new DatePicker(LocalDate.now()); 
		fieldFine = new TextField();
		fieldFine.setText(LocalTime.now().format(formatter));
		fieldFine.setPromptText("Inserire ora finale");
		centerPane.getChildren().addAll(pickerFine,fieldFine);
		this.setCenter(centerPane);
		//
		outputArea = new TextArea();
		outputArea.setEditable(false);
		outputArea.setPrefWidth(250);
		outputArea.setFont(Font.font("Courier New", FontWeight.BOLD, 12));
		outputArea.setText("");
		this.setRight(outputArea);
	}

	private void calcolaCostoNoleggio(ActionEvent event) {
		String citta = this.combo.getValue();
		if (citta==null) {
			Controller.alert("Errore", "Impossibile reperire la tariffa", "Non è stata selezionata alcuna città");
			this.outputArea.setText("");
			return;
		}
		DateTimeFormatter formatter = DateTimeFormatter.ofLocalizedTime(FormatStyle.SHORT).withLocale(Locale.ITALY);
		try {
			LocalTime oraInizioSosta = LocalTime.parse(fieldInizio.getText(), formatter);
			LocalTime oraFineSosta = LocalTime.parse(fieldFine.getText(), formatter );
			LocalDate dataInizioSosta = pickerInizio.getValue();
			LocalDate dataFineSosta = pickerFine.getValue();
			if (oraInizioSosta.isAfter(oraFineSosta) || dataInizioSosta.isAfter(dataFineSosta)) {
				Controller.alert("Errore", "Impossibile tornare indietro nel tempo", "La fine del noleggio precede l'inizio");
				this.outputArea.setText("");
				return;
			}
			double costo  = controller.getRentCost(citta, dataInizioSosta, oraInizioSosta, dataFineSosta, oraFineSosta); 
			NumberFormat currencyFormatter = NumberFormat.getCurrencyInstance(Locale.ITALY);
			this.outputArea.setText("Costo del noleggio " + currencyFormatter.format(costo));
		}
		catch (DateTimeParseException e){
			Controller.alert("Errore di formato", "Errore nel formato orario", "L'orario di inizio e fine noleggio deve avere la forma HH:MM");
			this.outputArea.setText("");
		}
	}


}
